<?php

declare(strict_types=1);

namespace Drupal\data_pipelines_sftp\Source\Resource;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\data_pipelines\Entity\DatasetInterface;
use Drupal\data_pipelines\Source\Resource\SourceResourceBase;
use Drupal\key\KeyRepositoryInterface;
use phpseclib3\Net\SFTP as SFTPClient;
use phpseclib3\Net\SFTP\Stream;

/**
 * Provides SFTP a source resource.
 */
final class Sftp extends SourceResourceBase {

  public function __construct(protected KeyRepositoryInterface $keyRepository) {}

  /**
   * {@inheritdoc}
   */
  public function getResource(DatasetInterface $dataset, string $field_name): mixed {
    $path = self::getFieldValue($dataset, $field_name);
    $host = self::getFieldValue($dataset, $field_name, 'host');
    $port = self::getFieldValue($dataset, $field_name, 'port');
    $credentials = self::getFieldValue($dataset, $field_name, 'credentials');
    ['username' => $username, 'password' => $password] = Json::decode($this->keyRepository->getKey($credentials)->getKeyValue());
    try {
      Stream::register();
      $sftp = new SFTPClient($host, $port ?: 22);
      if (!$sftp->login($username, $password)) {
        throw new \Exception("Authentication failed.");
      }
      if (!$resource = fopen("sftp://{$sftp}/{$path}", 'r')) {
        throw new \Exception('Resource not available.');
      }
      return $resource;
    }
    catch (\Exception $e) {
      $dataset->addLogMessage(sprintf("There was an error getting the file from the file server - %s", $e->getMessage()));
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getResourceBaseFieldDefinition(array $source_plugin_definition): BaseFieldDefinition {
    return BaseFieldDefinition::create('sftp')
      ->setLabel(new TranslatableMarkup('SFTP details'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'sftp',
      ]);
  }

}
